"""
Main interface for network-firewall service client

Usage::

    ```python
    import boto3
    from mypy_boto3_network_firewall import NetworkFirewallClient

    client: NetworkFirewallClient = boto3.client("network-firewall")
    ```
"""
import sys
from typing import Any, Dict, List, Type, overload

from botocore.client import ClientMeta

from mypy_boto3_network_firewall.paginator import (
    ListFirewallPoliciesPaginator,
    ListFirewallsPaginator,
    ListRuleGroupsPaginator,
    ListTagsForResourcePaginator,
)
from mypy_boto3_network_firewall.type_defs import (
    AssociateFirewallPolicyResponseTypeDef,
    AssociateSubnetsResponseTypeDef,
    CreateFirewallPolicyResponseTypeDef,
    CreateFirewallResponseTypeDef,
    CreateRuleGroupResponseTypeDef,
    DeleteFirewallPolicyResponseTypeDef,
    DeleteFirewallResponseTypeDef,
    DeleteRuleGroupResponseTypeDef,
    DescribeFirewallPolicyResponseTypeDef,
    DescribeFirewallResponseTypeDef,
    DescribeLoggingConfigurationResponseTypeDef,
    DescribeResourcePolicyResponseTypeDef,
    DescribeRuleGroupResponseTypeDef,
    DisassociateSubnetsResponseTypeDef,
    FirewallPolicyTypeDef,
    ListFirewallPoliciesResponseTypeDef,
    ListFirewallsResponseTypeDef,
    ListRuleGroupsResponseTypeDef,
    ListTagsForResourceResponseTypeDef,
    LoggingConfigurationTypeDef,
    RuleGroupTypeDef,
    SubnetMappingTypeDef,
    TagTypeDef,
    UpdateFirewallDeleteProtectionResponseTypeDef,
    UpdateFirewallDescriptionResponseTypeDef,
    UpdateFirewallPolicyChangeProtectionResponseTypeDef,
    UpdateFirewallPolicyResponseTypeDef,
    UpdateLoggingConfigurationResponseTypeDef,
    UpdateRuleGroupResponseTypeDef,
    UpdateSubnetChangeProtectionResponseTypeDef,
)

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal


__all__ = ("NetworkFirewallClient",)


class BotocoreClientError(BaseException):
    MSG_TEMPLATE: str

    def __init__(self, error_response: Dict[str, Any], operation_name: str) -> None:
        self.response: Dict[str, Any]
        self.operation_name: str


class Exceptions:
    ClientError: Type[BotocoreClientError]
    InsufficientCapacityException: Type[BotocoreClientError]
    InternalServerError: Type[BotocoreClientError]
    InvalidOperationException: Type[BotocoreClientError]
    InvalidRequestException: Type[BotocoreClientError]
    InvalidResourcePolicyException: Type[BotocoreClientError]
    InvalidTokenException: Type[BotocoreClientError]
    LimitExceededException: Type[BotocoreClientError]
    LogDestinationPermissionException: Type[BotocoreClientError]
    ResourceNotFoundException: Type[BotocoreClientError]
    ResourceOwnerCheckException: Type[BotocoreClientError]
    ThrottlingException: Type[BotocoreClientError]
    UnsupportedOperationException: Type[BotocoreClientError]


class NetworkFirewallClient:
    """
    [NetworkFirewall.Client documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client)
    """

    meta: ClientMeta
    exceptions: Exceptions

    def associate_firewall_policy(
        self,
        FirewallPolicyArn: str,
        UpdateToken: str = None,
        FirewallArn: str = None,
        FirewallName: str = None,
    ) -> AssociateFirewallPolicyResponseTypeDef:
        """
        [Client.associate_firewall_policy documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.associate_firewall_policy)
        """

    def associate_subnets(
        self,
        SubnetMappings: List["SubnetMappingTypeDef"],
        UpdateToken: str = None,
        FirewallArn: str = None,
        FirewallName: str = None,
    ) -> AssociateSubnetsResponseTypeDef:
        """
        [Client.associate_subnets documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.associate_subnets)
        """

    def can_paginate(self, operation_name: str) -> bool:
        """
        [Client.can_paginate documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.can_paginate)
        """

    def create_firewall(
        self,
        FirewallName: str,
        FirewallPolicyArn: str,
        VpcId: str,
        SubnetMappings: List["SubnetMappingTypeDef"],
        DeleteProtection: bool = None,
        SubnetChangeProtection: bool = None,
        FirewallPolicyChangeProtection: bool = None,
        Description: str = None,
        Tags: List["TagTypeDef"] = None,
    ) -> CreateFirewallResponseTypeDef:
        """
        [Client.create_firewall documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.create_firewall)
        """

    def create_firewall_policy(
        self,
        FirewallPolicyName: str,
        FirewallPolicy: "FirewallPolicyTypeDef",
        Description: str = None,
        Tags: List["TagTypeDef"] = None,
        DryRun: bool = None,
    ) -> CreateFirewallPolicyResponseTypeDef:
        """
        [Client.create_firewall_policy documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.create_firewall_policy)
        """

    def create_rule_group(
        self,
        RuleGroupName: str,
        Type: Literal["STATELESS", "STATEFUL"],
        Capacity: int,
        RuleGroup: "RuleGroupTypeDef" = None,
        Rules: str = None,
        Description: str = None,
        Tags: List["TagTypeDef"] = None,
        DryRun: bool = None,
    ) -> CreateRuleGroupResponseTypeDef:
        """
        [Client.create_rule_group documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.create_rule_group)
        """

    def delete_firewall(
        self, FirewallName: str = None, FirewallArn: str = None
    ) -> DeleteFirewallResponseTypeDef:
        """
        [Client.delete_firewall documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.delete_firewall)
        """

    def delete_firewall_policy(
        self, FirewallPolicyName: str = None, FirewallPolicyArn: str = None
    ) -> DeleteFirewallPolicyResponseTypeDef:
        """
        [Client.delete_firewall_policy documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.delete_firewall_policy)
        """

    def delete_resource_policy(self, ResourceArn: str) -> Dict[str, Any]:
        """
        [Client.delete_resource_policy documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.delete_resource_policy)
        """

    def delete_rule_group(
        self,
        RuleGroupName: str = None,
        RuleGroupArn: str = None,
        Type: Literal["STATELESS", "STATEFUL"] = None,
    ) -> DeleteRuleGroupResponseTypeDef:
        """
        [Client.delete_rule_group documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.delete_rule_group)
        """

    def describe_firewall(
        self, FirewallName: str = None, FirewallArn: str = None
    ) -> DescribeFirewallResponseTypeDef:
        """
        [Client.describe_firewall documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.describe_firewall)
        """

    def describe_firewall_policy(
        self, FirewallPolicyName: str = None, FirewallPolicyArn: str = None
    ) -> DescribeFirewallPolicyResponseTypeDef:
        """
        [Client.describe_firewall_policy documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.describe_firewall_policy)
        """

    def describe_logging_configuration(
        self, FirewallArn: str = None, FirewallName: str = None
    ) -> DescribeLoggingConfigurationResponseTypeDef:
        """
        [Client.describe_logging_configuration documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.describe_logging_configuration)
        """

    def describe_resource_policy(self, ResourceArn: str) -> DescribeResourcePolicyResponseTypeDef:
        """
        [Client.describe_resource_policy documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.describe_resource_policy)
        """

    def describe_rule_group(
        self,
        RuleGroupName: str = None,
        RuleGroupArn: str = None,
        Type: Literal["STATELESS", "STATEFUL"] = None,
    ) -> DescribeRuleGroupResponseTypeDef:
        """
        [Client.describe_rule_group documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.describe_rule_group)
        """

    def disassociate_subnets(
        self,
        SubnetIds: List[str],
        UpdateToken: str = None,
        FirewallArn: str = None,
        FirewallName: str = None,
    ) -> DisassociateSubnetsResponseTypeDef:
        """
        [Client.disassociate_subnets documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.disassociate_subnets)
        """

    def generate_presigned_url(
        self,
        ClientMethod: str,
        Params: Dict[str, Any] = None,
        ExpiresIn: int = 3600,
        HttpMethod: str = None,
    ) -> str:
        """
        [Client.generate_presigned_url documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.generate_presigned_url)
        """

    def list_firewall_policies(
        self, NextToken: str = None, MaxResults: int = None
    ) -> ListFirewallPoliciesResponseTypeDef:
        """
        [Client.list_firewall_policies documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.list_firewall_policies)
        """

    def list_firewalls(
        self, NextToken: str = None, VpcIds: List[str] = None, MaxResults: int = None
    ) -> ListFirewallsResponseTypeDef:
        """
        [Client.list_firewalls documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.list_firewalls)
        """

    def list_rule_groups(
        self, NextToken: str = None, MaxResults: int = None
    ) -> ListRuleGroupsResponseTypeDef:
        """
        [Client.list_rule_groups documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.list_rule_groups)
        """

    def list_tags_for_resource(
        self, ResourceArn: str, NextToken: str = None, MaxResults: int = None
    ) -> ListTagsForResourceResponseTypeDef:
        """
        [Client.list_tags_for_resource documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.list_tags_for_resource)
        """

    def put_resource_policy(self, ResourceArn: str, Policy: str) -> Dict[str, Any]:
        """
        [Client.put_resource_policy documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.put_resource_policy)
        """

    def tag_resource(self, ResourceArn: str, Tags: List["TagTypeDef"]) -> Dict[str, Any]:
        """
        [Client.tag_resource documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.tag_resource)
        """

    def untag_resource(self, ResourceArn: str, TagKeys: List[str]) -> Dict[str, Any]:
        """
        [Client.untag_resource documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.untag_resource)
        """

    def update_firewall_delete_protection(
        self,
        DeleteProtection: bool,
        UpdateToken: str = None,
        FirewallArn: str = None,
        FirewallName: str = None,
    ) -> UpdateFirewallDeleteProtectionResponseTypeDef:
        """
        [Client.update_firewall_delete_protection documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.update_firewall_delete_protection)
        """

    def update_firewall_description(
        self,
        UpdateToken: str = None,
        FirewallArn: str = None,
        FirewallName: str = None,
        Description: str = None,
    ) -> UpdateFirewallDescriptionResponseTypeDef:
        """
        [Client.update_firewall_description documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.update_firewall_description)
        """

    def update_firewall_policy(
        self,
        UpdateToken: str,
        FirewallPolicy: "FirewallPolicyTypeDef",
        FirewallPolicyArn: str = None,
        FirewallPolicyName: str = None,
        Description: str = None,
        DryRun: bool = None,
    ) -> UpdateFirewallPolicyResponseTypeDef:
        """
        [Client.update_firewall_policy documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.update_firewall_policy)
        """

    def update_firewall_policy_change_protection(
        self,
        FirewallPolicyChangeProtection: bool,
        UpdateToken: str = None,
        FirewallArn: str = None,
        FirewallName: str = None,
    ) -> UpdateFirewallPolicyChangeProtectionResponseTypeDef:
        """
        [Client.update_firewall_policy_change_protection documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.update_firewall_policy_change_protection)
        """

    def update_logging_configuration(
        self,
        FirewallArn: str = None,
        FirewallName: str = None,
        LoggingConfiguration: "LoggingConfigurationTypeDef" = None,
    ) -> UpdateLoggingConfigurationResponseTypeDef:
        """
        [Client.update_logging_configuration documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.update_logging_configuration)
        """

    def update_rule_group(
        self,
        UpdateToken: str,
        RuleGroupArn: str = None,
        RuleGroupName: str = None,
        RuleGroup: "RuleGroupTypeDef" = None,
        Rules: str = None,
        Type: Literal["STATELESS", "STATEFUL"] = None,
        Description: str = None,
        DryRun: bool = None,
    ) -> UpdateRuleGroupResponseTypeDef:
        """
        [Client.update_rule_group documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.update_rule_group)
        """

    def update_subnet_change_protection(
        self,
        SubnetChangeProtection: bool,
        UpdateToken: str = None,
        FirewallArn: str = None,
        FirewallName: str = None,
    ) -> UpdateSubnetChangeProtectionResponseTypeDef:
        """
        [Client.update_subnet_change_protection documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Client.update_subnet_change_protection)
        """

    @overload
    def get_paginator(
        self, operation_name: Literal["list_firewall_policies"]
    ) -> ListFirewallPoliciesPaginator:
        """
        [Paginator.ListFirewallPolicies documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Paginator.ListFirewallPolicies)
        """

    @overload
    def get_paginator(self, operation_name: Literal["list_firewalls"]) -> ListFirewallsPaginator:
        """
        [Paginator.ListFirewalls documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Paginator.ListFirewalls)
        """

    @overload
    def get_paginator(self, operation_name: Literal["list_rule_groups"]) -> ListRuleGroupsPaginator:
        """
        [Paginator.ListRuleGroups documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Paginator.ListRuleGroups)
        """

    @overload
    def get_paginator(
        self, operation_name: Literal["list_tags_for_resource"]
    ) -> ListTagsForResourcePaginator:
        """
        [Paginator.ListTagsForResource documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Paginator.ListTagsForResource)
        """



"""
Main interface for network-firewall service type definitions.

Usage::

    ```python
    from mypy_boto3_network_firewall.type_defs import ActionDefinitionTypeDef

    data: ActionDefinitionTypeDef = {...}
    ```
"""
import sys
from typing import Dict, List

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal
if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict


__all__ = (
    "ActionDefinitionTypeDef",
    "AddressTypeDef",
    "AttachmentTypeDef",
    "CustomActionTypeDef",
    "DimensionTypeDef",
    "FirewallMetadataTypeDef",
    "FirewallPolicyMetadataTypeDef",
    "FirewallPolicyResponseTypeDef",
    "FirewallPolicyTypeDef",
    "FirewallStatusTypeDef",
    "FirewallTypeDef",
    "HeaderTypeDef",
    "IPSetTypeDef",
    "LogDestinationConfigTypeDef",
    "LoggingConfigurationTypeDef",
    "MatchAttributesTypeDef",
    "PerObjectStatusTypeDef",
    "PortRangeTypeDef",
    "PortSetTypeDef",
    "PublishMetricActionTypeDef",
    "RuleDefinitionTypeDef",
    "RuleGroupMetadataTypeDef",
    "RuleGroupResponseTypeDef",
    "RuleGroupTypeDef",
    "RuleOptionTypeDef",
    "RuleVariablesTypeDef",
    "RulesSourceListTypeDef",
    "RulesSourceTypeDef",
    "StatefulRuleGroupReferenceTypeDef",
    "StatefulRuleTypeDef",
    "StatelessRuleGroupReferenceTypeDef",
    "StatelessRuleTypeDef",
    "StatelessRulesAndCustomActionsTypeDef",
    "SubnetMappingTypeDef",
    "SyncStateTypeDef",
    "TCPFlagFieldTypeDef",
    "TagTypeDef",
    "AssociateFirewallPolicyResponseTypeDef",
    "AssociateSubnetsResponseTypeDef",
    "CreateFirewallPolicyResponseTypeDef",
    "CreateFirewallResponseTypeDef",
    "CreateRuleGroupResponseTypeDef",
    "DeleteFirewallPolicyResponseTypeDef",
    "DeleteFirewallResponseTypeDef",
    "DeleteRuleGroupResponseTypeDef",
    "DescribeFirewallPolicyResponseTypeDef",
    "DescribeFirewallResponseTypeDef",
    "DescribeLoggingConfigurationResponseTypeDef",
    "DescribeResourcePolicyResponseTypeDef",
    "DescribeRuleGroupResponseTypeDef",
    "DisassociateSubnetsResponseTypeDef",
    "ListFirewallPoliciesResponseTypeDef",
    "ListFirewallsResponseTypeDef",
    "ListRuleGroupsResponseTypeDef",
    "ListTagsForResourceResponseTypeDef",
    "PaginatorConfigTypeDef",
    "UpdateFirewallDeleteProtectionResponseTypeDef",
    "UpdateFirewallDescriptionResponseTypeDef",
    "UpdateFirewallPolicyChangeProtectionResponseTypeDef",
    "UpdateFirewallPolicyResponseTypeDef",
    "UpdateLoggingConfigurationResponseTypeDef",
    "UpdateRuleGroupResponseTypeDef",
    "UpdateSubnetChangeProtectionResponseTypeDef",
)

ActionDefinitionTypeDef = TypedDict(
    "ActionDefinitionTypeDef", {"PublishMetricAction": "PublishMetricActionTypeDef"}, total=False
)

AddressTypeDef = TypedDict("AddressTypeDef", {"AddressDefinition": str})

AttachmentTypeDef = TypedDict(
    "AttachmentTypeDef",
    {
        "SubnetId": str,
        "EndpointId": str,
        "Status": Literal["CREATING", "DELETING", "SCALING", "READY"],
    },
    total=False,
)

CustomActionTypeDef = TypedDict(
    "CustomActionTypeDef", {"ActionName": str, "ActionDefinition": "ActionDefinitionTypeDef"}
)

DimensionTypeDef = TypedDict("DimensionTypeDef", {"Value": str})

FirewallMetadataTypeDef = TypedDict(
    "FirewallMetadataTypeDef", {"FirewallName": str, "FirewallArn": str}, total=False
)

FirewallPolicyMetadataTypeDef = TypedDict(
    "FirewallPolicyMetadataTypeDef", {"Name": str, "Arn": str}, total=False
)

_RequiredFirewallPolicyResponseTypeDef = TypedDict(
    "_RequiredFirewallPolicyResponseTypeDef",
    {"FirewallPolicyName": str, "FirewallPolicyArn": str, "FirewallPolicyId": str},
)
_OptionalFirewallPolicyResponseTypeDef = TypedDict(
    "_OptionalFirewallPolicyResponseTypeDef",
    {
        "Description": str,
        "FirewallPolicyStatus": Literal["ACTIVE", "DELETING"],
        "Tags": List["TagTypeDef"],
    },
    total=False,
)


class FirewallPolicyResponseTypeDef(
    _RequiredFirewallPolicyResponseTypeDef, _OptionalFirewallPolicyResponseTypeDef
):
    pass


_RequiredFirewallPolicyTypeDef = TypedDict(
    "_RequiredFirewallPolicyTypeDef",
    {"StatelessDefaultActions": List[str], "StatelessFragmentDefaultActions": List[str]},
)
_OptionalFirewallPolicyTypeDef = TypedDict(
    "_OptionalFirewallPolicyTypeDef",
    {
        "StatelessRuleGroupReferences": List["StatelessRuleGroupReferenceTypeDef"],
        "StatelessCustomActions": List["CustomActionTypeDef"],
        "StatefulRuleGroupReferences": List["StatefulRuleGroupReferenceTypeDef"],
    },
    total=False,
)


class FirewallPolicyTypeDef(_RequiredFirewallPolicyTypeDef, _OptionalFirewallPolicyTypeDef):
    pass


_RequiredFirewallStatusTypeDef = TypedDict(
    "_RequiredFirewallStatusTypeDef",
    {
        "Status": Literal["PROVISIONING", "DELETING", "READY"],
        "ConfigurationSyncStateSummary": Literal["PENDING", "IN_SYNC"],
    },
)
_OptionalFirewallStatusTypeDef = TypedDict(
    "_OptionalFirewallStatusTypeDef", {"SyncStates": Dict[str, "SyncStateTypeDef"]}, total=False
)


class FirewallStatusTypeDef(_RequiredFirewallStatusTypeDef, _OptionalFirewallStatusTypeDef):
    pass


_RequiredFirewallTypeDef = TypedDict(
    "_RequiredFirewallTypeDef",
    {
        "FirewallPolicyArn": str,
        "VpcId": str,
        "SubnetMappings": List["SubnetMappingTypeDef"],
        "FirewallId": str,
    },
)
_OptionalFirewallTypeDef = TypedDict(
    "_OptionalFirewallTypeDef",
    {
        "FirewallName": str,
        "FirewallArn": str,
        "DeleteProtection": bool,
        "SubnetChangeProtection": bool,
        "FirewallPolicyChangeProtection": bool,
        "Description": str,
        "Tags": List["TagTypeDef"],
    },
    total=False,
)


class FirewallTypeDef(_RequiredFirewallTypeDef, _OptionalFirewallTypeDef):
    pass


HeaderTypeDef = TypedDict(
    "HeaderTypeDef",
    {
        "Protocol": Literal[
            "IP",
            "TCP",
            "UDP",
            "ICMP",
            "HTTP",
            "FTP",
            "TLS",
            "SMB",
            "DNS",
            "DCERPC",
            "SSH",
            "SMTP",
            "IMAP",
            "MSN",
            "KRB5",
            "IKEV2",
            "TFTP",
            "NTP",
            "DHCP",
        ],
        "Source": str,
        "SourcePort": str,
        "Direction": Literal["FORWARD", "ANY"],
        "Destination": str,
        "DestinationPort": str,
    },
)

IPSetTypeDef = TypedDict("IPSetTypeDef", {"Definition": List[str]})

LogDestinationConfigTypeDef = TypedDict(
    "LogDestinationConfigTypeDef",
    {
        "LogType": Literal["ALERT", "FLOW"],
        "LogDestinationType": Literal["S3", "CloudWatchLogs", "KinesisDataFirehose"],
        "LogDestination": Dict[str, str],
    },
)

LoggingConfigurationTypeDef = TypedDict(
    "LoggingConfigurationTypeDef", {"LogDestinationConfigs": List["LogDestinationConfigTypeDef"]}
)

MatchAttributesTypeDef = TypedDict(
    "MatchAttributesTypeDef",
    {
        "Sources": List["AddressTypeDef"],
        "Destinations": List["AddressTypeDef"],
        "SourcePorts": List["PortRangeTypeDef"],
        "DestinationPorts": List["PortRangeTypeDef"],
        "Protocols": List[int],
        "TCPFlags": List["TCPFlagFieldTypeDef"],
    },
    total=False,
)

PerObjectStatusTypeDef = TypedDict(
    "PerObjectStatusTypeDef",
    {"SyncStatus": Literal["PENDING", "IN_SYNC"], "UpdateToken": str},
    total=False,
)

PortRangeTypeDef = TypedDict("PortRangeTypeDef", {"FromPort": int, "ToPort": int})

PortSetTypeDef = TypedDict("PortSetTypeDef", {"Definition": List[str]}, total=False)

PublishMetricActionTypeDef = TypedDict(
    "PublishMetricActionTypeDef", {"Dimensions": List["DimensionTypeDef"]}
)

RuleDefinitionTypeDef = TypedDict(
    "RuleDefinitionTypeDef", {"MatchAttributes": "MatchAttributesTypeDef", "Actions": List[str]}
)

RuleGroupMetadataTypeDef = TypedDict(
    "RuleGroupMetadataTypeDef", {"Name": str, "Arn": str}, total=False
)

_RequiredRuleGroupResponseTypeDef = TypedDict(
    "_RequiredRuleGroupResponseTypeDef",
    {"RuleGroupArn": str, "RuleGroupName": str, "RuleGroupId": str},
)
_OptionalRuleGroupResponseTypeDef = TypedDict(
    "_OptionalRuleGroupResponseTypeDef",
    {
        "Description": str,
        "Type": Literal["STATELESS", "STATEFUL"],
        "Capacity": int,
        "RuleGroupStatus": Literal["ACTIVE", "DELETING"],
        "Tags": List["TagTypeDef"],
    },
    total=False,
)


class RuleGroupResponseTypeDef(
    _RequiredRuleGroupResponseTypeDef, _OptionalRuleGroupResponseTypeDef
):
    pass


_RequiredRuleGroupTypeDef = TypedDict(
    "_RequiredRuleGroupTypeDef", {"RulesSource": "RulesSourceTypeDef"}
)
_OptionalRuleGroupTypeDef = TypedDict(
    "_OptionalRuleGroupTypeDef", {"RuleVariables": "RuleVariablesTypeDef"}, total=False
)


class RuleGroupTypeDef(_RequiredRuleGroupTypeDef, _OptionalRuleGroupTypeDef):
    pass


_RequiredRuleOptionTypeDef = TypedDict("_RequiredRuleOptionTypeDef", {"Keyword": str})
_OptionalRuleOptionTypeDef = TypedDict(
    "_OptionalRuleOptionTypeDef", {"Settings": List[str]}, total=False
)


class RuleOptionTypeDef(_RequiredRuleOptionTypeDef, _OptionalRuleOptionTypeDef):
    pass


RuleVariablesTypeDef = TypedDict(
    "RuleVariablesTypeDef",
    {"IPSets": Dict[str, "IPSetTypeDef"], "PortSets": Dict[str, "PortSetTypeDef"]},
    total=False,
)

RulesSourceListTypeDef = TypedDict(
    "RulesSourceListTypeDef",
    {
        "Targets": List[str],
        "TargetTypes": List[Literal["TLS_SNI", "HTTP_HOST"]],
        "GeneratedRulesType": Literal["ALLOWLIST", "DENYLIST"],
    },
)

RulesSourceTypeDef = TypedDict(
    "RulesSourceTypeDef",
    {
        "RulesString": str,
        "RulesSourceList": "RulesSourceListTypeDef",
        "StatefulRules": List["StatefulRuleTypeDef"],
        "StatelessRulesAndCustomActions": "StatelessRulesAndCustomActionsTypeDef",
    },
    total=False,
)

StatefulRuleGroupReferenceTypeDef = TypedDict(
    "StatefulRuleGroupReferenceTypeDef", {"ResourceArn": str}
)

StatefulRuleTypeDef = TypedDict(
    "StatefulRuleTypeDef",
    {
        "Action": Literal["PASS", "DROP", "ALERT"],
        "Header": "HeaderTypeDef",
        "RuleOptions": List["RuleOptionTypeDef"],
    },
)

StatelessRuleGroupReferenceTypeDef = TypedDict(
    "StatelessRuleGroupReferenceTypeDef", {"ResourceArn": str, "Priority": int}
)

StatelessRuleTypeDef = TypedDict(
    "StatelessRuleTypeDef", {"RuleDefinition": "RuleDefinitionTypeDef", "Priority": int}
)

_RequiredStatelessRulesAndCustomActionsTypeDef = TypedDict(
    "_RequiredStatelessRulesAndCustomActionsTypeDef",
    {"StatelessRules": List["StatelessRuleTypeDef"]},
)
_OptionalStatelessRulesAndCustomActionsTypeDef = TypedDict(
    "_OptionalStatelessRulesAndCustomActionsTypeDef",
    {"CustomActions": List["CustomActionTypeDef"]},
    total=False,
)


class StatelessRulesAndCustomActionsTypeDef(
    _RequiredStatelessRulesAndCustomActionsTypeDef, _OptionalStatelessRulesAndCustomActionsTypeDef
):
    pass


SubnetMappingTypeDef = TypedDict("SubnetMappingTypeDef", {"SubnetId": str})

SyncStateTypeDef = TypedDict(
    "SyncStateTypeDef",
    {"Attachment": "AttachmentTypeDef", "Config": Dict[str, "PerObjectStatusTypeDef"]},
    total=False,
)

_RequiredTCPFlagFieldTypeDef = TypedDict(
    "_RequiredTCPFlagFieldTypeDef",
    {"Flags": List[Literal["FIN", "SYN", "RST", "PSH", "ACK", "URG", "ECE", "CWR"]]},
)
_OptionalTCPFlagFieldTypeDef = TypedDict(
    "_OptionalTCPFlagFieldTypeDef",
    {"Masks": List[Literal["FIN", "SYN", "RST", "PSH", "ACK", "URG", "ECE", "CWR"]]},
    total=False,
)


class TCPFlagFieldTypeDef(_RequiredTCPFlagFieldTypeDef, _OptionalTCPFlagFieldTypeDef):
    pass


TagTypeDef = TypedDict("TagTypeDef", {"Key": str, "Value": str})

AssociateFirewallPolicyResponseTypeDef = TypedDict(
    "AssociateFirewallPolicyResponseTypeDef",
    {"FirewallArn": str, "FirewallName": str, "FirewallPolicyArn": str, "UpdateToken": str},
    total=False,
)

AssociateSubnetsResponseTypeDef = TypedDict(
    "AssociateSubnetsResponseTypeDef",
    {
        "FirewallArn": str,
        "FirewallName": str,
        "SubnetMappings": List["SubnetMappingTypeDef"],
        "UpdateToken": str,
    },
    total=False,
)

CreateFirewallPolicyResponseTypeDef = TypedDict(
    "CreateFirewallPolicyResponseTypeDef",
    {"UpdateToken": str, "FirewallPolicyResponse": "FirewallPolicyResponseTypeDef"},
)

CreateFirewallResponseTypeDef = TypedDict(
    "CreateFirewallResponseTypeDef",
    {"Firewall": "FirewallTypeDef", "FirewallStatus": "FirewallStatusTypeDef"},
    total=False,
)

CreateRuleGroupResponseTypeDef = TypedDict(
    "CreateRuleGroupResponseTypeDef",
    {"UpdateToken": str, "RuleGroupResponse": "RuleGroupResponseTypeDef"},
)

DeleteFirewallPolicyResponseTypeDef = TypedDict(
    "DeleteFirewallPolicyResponseTypeDef",
    {"FirewallPolicyResponse": "FirewallPolicyResponseTypeDef"},
)

DeleteFirewallResponseTypeDef = TypedDict(
    "DeleteFirewallResponseTypeDef",
    {"Firewall": "FirewallTypeDef", "FirewallStatus": "FirewallStatusTypeDef"},
    total=False,
)

DeleteRuleGroupResponseTypeDef = TypedDict(
    "DeleteRuleGroupResponseTypeDef", {"RuleGroupResponse": "RuleGroupResponseTypeDef"}
)

_RequiredDescribeFirewallPolicyResponseTypeDef = TypedDict(
    "_RequiredDescribeFirewallPolicyResponseTypeDef",
    {"UpdateToken": str, "FirewallPolicyResponse": "FirewallPolicyResponseTypeDef"},
)
_OptionalDescribeFirewallPolicyResponseTypeDef = TypedDict(
    "_OptionalDescribeFirewallPolicyResponseTypeDef",
    {"FirewallPolicy": "FirewallPolicyTypeDef"},
    total=False,
)


class DescribeFirewallPolicyResponseTypeDef(
    _RequiredDescribeFirewallPolicyResponseTypeDef, _OptionalDescribeFirewallPolicyResponseTypeDef
):
    pass


DescribeFirewallResponseTypeDef = TypedDict(
    "DescribeFirewallResponseTypeDef",
    {"UpdateToken": str, "Firewall": "FirewallTypeDef", "FirewallStatus": "FirewallStatusTypeDef"},
    total=False,
)

DescribeLoggingConfigurationResponseTypeDef = TypedDict(
    "DescribeLoggingConfigurationResponseTypeDef",
    {"FirewallArn": str, "LoggingConfiguration": "LoggingConfigurationTypeDef"},
    total=False,
)

DescribeResourcePolicyResponseTypeDef = TypedDict(
    "DescribeResourcePolicyResponseTypeDef", {"Policy": str}, total=False
)

_RequiredDescribeRuleGroupResponseTypeDef = TypedDict(
    "_RequiredDescribeRuleGroupResponseTypeDef",
    {"UpdateToken": str, "RuleGroupResponse": "RuleGroupResponseTypeDef"},
)
_OptionalDescribeRuleGroupResponseTypeDef = TypedDict(
    "_OptionalDescribeRuleGroupResponseTypeDef", {"RuleGroup": "RuleGroupTypeDef"}, total=False
)


class DescribeRuleGroupResponseTypeDef(
    _RequiredDescribeRuleGroupResponseTypeDef, _OptionalDescribeRuleGroupResponseTypeDef
):
    pass


DisassociateSubnetsResponseTypeDef = TypedDict(
    "DisassociateSubnetsResponseTypeDef",
    {
        "FirewallArn": str,
        "FirewallName": str,
        "SubnetMappings": List["SubnetMappingTypeDef"],
        "UpdateToken": str,
    },
    total=False,
)

ListFirewallPoliciesResponseTypeDef = TypedDict(
    "ListFirewallPoliciesResponseTypeDef",
    {"NextToken": str, "FirewallPolicies": List["FirewallPolicyMetadataTypeDef"]},
    total=False,
)

ListFirewallsResponseTypeDef = TypedDict(
    "ListFirewallsResponseTypeDef",
    {"NextToken": str, "Firewalls": List["FirewallMetadataTypeDef"]},
    total=False,
)

ListRuleGroupsResponseTypeDef = TypedDict(
    "ListRuleGroupsResponseTypeDef",
    {"NextToken": str, "RuleGroups": List["RuleGroupMetadataTypeDef"]},
    total=False,
)

ListTagsForResourceResponseTypeDef = TypedDict(
    "ListTagsForResourceResponseTypeDef",
    {"NextToken": str, "Tags": List["TagTypeDef"]},
    total=False,
)

PaginatorConfigTypeDef = TypedDict(
    "PaginatorConfigTypeDef", {"MaxItems": int, "PageSize": int, "StartingToken": str}, total=False
)

UpdateFirewallDeleteProtectionResponseTypeDef = TypedDict(
    "UpdateFirewallDeleteProtectionResponseTypeDef",
    {"FirewallArn": str, "FirewallName": str, "DeleteProtection": bool, "UpdateToken": str},
    total=False,
)

UpdateFirewallDescriptionResponseTypeDef = TypedDict(
    "UpdateFirewallDescriptionResponseTypeDef",
    {"FirewallArn": str, "FirewallName": str, "Description": str, "UpdateToken": str},
    total=False,
)

UpdateFirewallPolicyChangeProtectionResponseTypeDef = TypedDict(
    "UpdateFirewallPolicyChangeProtectionResponseTypeDef",
    {
        "UpdateToken": str,
        "FirewallArn": str,
        "FirewallName": str,
        "FirewallPolicyChangeProtection": bool,
    },
    total=False,
)

UpdateFirewallPolicyResponseTypeDef = TypedDict(
    "UpdateFirewallPolicyResponseTypeDef",
    {"UpdateToken": str, "FirewallPolicyResponse": "FirewallPolicyResponseTypeDef"},
)

UpdateLoggingConfigurationResponseTypeDef = TypedDict(
    "UpdateLoggingConfigurationResponseTypeDef",
    {
        "FirewallArn": str,
        "FirewallName": str,
        "LoggingConfiguration": "LoggingConfigurationTypeDef",
    },
    total=False,
)

UpdateRuleGroupResponseTypeDef = TypedDict(
    "UpdateRuleGroupResponseTypeDef",
    {"UpdateToken": str, "RuleGroupResponse": "RuleGroupResponseTypeDef"},
)

UpdateSubnetChangeProtectionResponseTypeDef = TypedDict(
    "UpdateSubnetChangeProtectionResponseTypeDef",
    {"UpdateToken": str, "FirewallArn": str, "FirewallName": str, "SubnetChangeProtection": bool},
    total=False,
)



"""
Main interface for network-firewall service client paginators.

Usage::

    ```python
    import boto3

    from mypy_boto3_network_firewall import NetworkFirewallClient
    from mypy_boto3_network_firewall.paginator import (
        ListFirewallPoliciesPaginator,
        ListFirewallsPaginator,
        ListRuleGroupsPaginator,
        ListTagsForResourcePaginator,
    )

    client: NetworkFirewallClient = boto3.client("network-firewall")

    list_firewall_policies_paginator: ListFirewallPoliciesPaginator = client.get_paginator("list_firewall_policies")
    list_firewalls_paginator: ListFirewallsPaginator = client.get_paginator("list_firewalls")
    list_rule_groups_paginator: ListRuleGroupsPaginator = client.get_paginator("list_rule_groups")
    list_tags_for_resource_paginator: ListTagsForResourcePaginator = client.get_paginator("list_tags_for_resource")
    ```
"""
from typing import Iterator, List

from botocore.paginate import Paginator as Boto3Paginator

from mypy_boto3_network_firewall.type_defs import (
    ListFirewallPoliciesResponseTypeDef,
    ListFirewallsResponseTypeDef,
    ListRuleGroupsResponseTypeDef,
    ListTagsForResourceResponseTypeDef,
    PaginatorConfigTypeDef,
)

__all__ = (
    "ListFirewallPoliciesPaginator",
    "ListFirewallsPaginator",
    "ListRuleGroupsPaginator",
    "ListTagsForResourcePaginator",
)


class ListFirewallPoliciesPaginator(Boto3Paginator):
    """
    [Paginator.ListFirewallPolicies documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Paginator.ListFirewallPolicies)
    """

    def paginate(
        self, PaginationConfig: PaginatorConfigTypeDef = None
    ) -> Iterator[ListFirewallPoliciesResponseTypeDef]:
        """
        [ListFirewallPolicies.paginate documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Paginator.ListFirewallPolicies.paginate)
        """


class ListFirewallsPaginator(Boto3Paginator):
    """
    [Paginator.ListFirewalls documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Paginator.ListFirewalls)
    """

    def paginate(
        self, VpcIds: List[str] = None, PaginationConfig: PaginatorConfigTypeDef = None
    ) -> Iterator[ListFirewallsResponseTypeDef]:
        """
        [ListFirewalls.paginate documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Paginator.ListFirewalls.paginate)
        """


class ListRuleGroupsPaginator(Boto3Paginator):
    """
    [Paginator.ListRuleGroups documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Paginator.ListRuleGroups)
    """

    def paginate(
        self, PaginationConfig: PaginatorConfigTypeDef = None
    ) -> Iterator[ListRuleGroupsResponseTypeDef]:
        """
        [ListRuleGroups.paginate documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Paginator.ListRuleGroups.paginate)
        """


class ListTagsForResourcePaginator(Boto3Paginator):
    """
    [Paginator.ListTagsForResource documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Paginator.ListTagsForResource)
    """

    def paginate(
        self, ResourceArn: str, PaginationConfig: PaginatorConfigTypeDef = None
    ) -> Iterator[ListTagsForResourceResponseTypeDef]:
        """
        [ListTagsForResource.paginate documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.44/reference/services/network-firewall.html#NetworkFirewall.Paginator.ListTagsForResource.paginate)
        """
